
//------------------------------------------------<                   axlib v1.1                      >----------------------------------------------------
//------------------------------------------------<      MH-BUS  >----------------------------------------------------
//------------------------------------------------<      2015 http://www.avrki.ru      >----------------------------------------------------

#ifndef MHBUS_SLAVE_H_
#define MHBUS_SLAVE_H_

#if !defined(MAIN_INIT_H_)
#error "You must included (#include \"main_init.h\") befor use (#include <axlib/mhbus_slave.h>)."
#endif

//-------------------------------------------------------------------------
//						 
//-------------------------------------------------------------------------

#include <avr/eeprom.h>
#include <axlib/type_var.h>
#include <axlib/usart.h>
#include <axlib/mhbus_const.h>

//-------------------------------------------------------------------------
//							
//-------------------------------------------------------------------------

#if defined(MH_BUS_RS485_ON)
	#define RS_485_INIT()		(RS_485_DDR |= (1 << RS_485_PIN))
	#define RS_485_IN			(RS_485_PORT &= ~(1 << RS_485_PIN))
	#define RS_485_OUT			(RS_485_PORT |= (1 << RS_485_PIN))
	#define RS_485_TIME_OUT		10
#endif

//-------------------------------------------------------------------------
//						 
//------------------------------------------------------------------------

volatile BYTE eeprom_add EEMEM;		//   
volatile BYTE mh_data[8] = {0};		//     
	
#define MH_BUS_ADD		mh_data[0]		//   
#define MH_BUS_COM		mh_data[1]		//    
#define MH_BUS_REG		mh_data[2]		//  
#define MH_BUS_BYTE_1	mh_data[3]		//   
#define MH_BUS_BYTE_2	mh_data[4]		//   
#define MH_BUS_BYTE_3	mh_data[5]		//   
#define MH_BUS_BYTE_4	mh_data[6]		//   

//-------------------------------------------------------------------------
//	  MH-BUS
//
//-------------------------------------------------------------------------

void mhbus_init(void)
{	
	usart_init(MH_BUS_USART_BOD);

//--------------------------------------------------------
#if defined(MH_BUS_RS485_ON)
	RS_485_INIT();
	RS_485_IN;
#endif
}
//--------------------------------------------------------

//-------------------------------------------------------------------------
//	   
//
//	 :
//
//		BYTE *data        
//
//		 TRUE      FALSE   
//         .
//-------------------------------------------------------------------------

BYTE mhbus_sum(BYTE *data)
{
	WORD compare = 0;
	WORD crc = 0;
	BYTE i = 0;
	
	while (i < 7)
		{
			compare += *data;
			data++;
			i++;
		}
	
	crc = *data;
	crc = (crc << 8);
	data++;
	crc |= *data;

	if(crc != compare) return FALSE;
	return TRUE;
}

//-------------------------------------------------------------------------
//	  
//
//	 :
//
//		BYTE *data       
//
//	 .
//
//		OK				 ,   
//		STOP_FAIL		  
//		START_FAIL		  
//		CON_SUM_FAIL	  
//		COM_FAIL		 
//
//-------------------------------------------------------------------------

BYTE mhbus_test(BYTE *arr)
{
	BYTE arr_data[9];
	UBYTE i = 0;
	
	if(*arr != MH_BUS_START_BYTE) return MH_BUS_START_FAIL;
	arr++;
	
	for (i=0; i<9; i++)
		{
			arr_data[i] = *arr;
			arr++;
		}
	
	if(!(mhbus_sum(arr_data))) return MH_BUS_CON_SUM_FAIL;
	if(arr_data[1] < 0x21) return MH_BUS_COM_FAIL;
	if(*arr != MH_BUS_STOP_BYTE) return MH_BUS_STOP_FAIL;
	
	return MH_BUS_OK;
}

//-------------------------------------------------------------------------
//	    .
//
//	  WORD timeout      .
//   FALSE          .
//   OK          
//
//-------------------------------------------------------------------------

#if defined(MH_BUS_SLAVE)	//  
	BYTE mhbus_read(BYTE test_add, BYTE timeout)
#endif

#if defined(MH_BUS_MASTER)	//  
	BYTE mhbus_read(WORD timeout)
#endif
{
	
	BYTE temp[11];
	BYTE answer = FALSE;

	while ((!uart_flag) && timeout)
		{
			timeout = timeout - 1;
			_delay_ms(1);
		}
	
	_delay_ms(50);
	
	answer = usart_data();
	
	if(!answer) 
		{
			return MH_BUS_MOD_FAIL;
		}
	
	answer = usart_str_in(temp);
	
	if(answer != 11) return MH_BUS_FAIL_DATA;
	
	answer = mhbus_test(temp);
	
	if(answer != MH_BUS_OK)
		{
			return answer;
		}

//--------------------------------------------------------
#if defined(MH_BUS_SLAVE)	//  
	if(!((temp[1] == test_add) | (temp[1] == MH_BUS_ALL_ADD)))
		{
			return FALSE;
		}
#endif
//--------------------------------------------------------

	for(BYTE i=0; i<8; i++)
		{
			mh_data[i] = temp[(i+1)];
		}
	mh_data[7] = 0x00;
	
	return MH_BUS_OK;
}

//-------------------------------------------------------------------------
//	    .
//
//	 .
//
//		BYTE com -  
//		BYTE reg -  
//		BYTE *str -    .
//					    ,    
//					    .
//-------------------------------------------------------------------------

//--------------------------------------------------------
#if defined(MH_BUS_SLAVE)	//  
void mhbus_write(BYTE com, BYTE reg, BYTE *str)
{
	BYTE test_add = eeprom_read_byte(&eeprom_add);
#endif
#if defined(MH_BUS_MASTER)	//  
void mhbus_write(BYTE add, BYTE com, BYTE reg, BYTE *str)
{
#endif
//-------------------------------------------------------
	
	BYTE str_out[11];
	WORD crc = 0;
	
	str_out[0] = 0xAA;
	str_out[10] = 0x55;
	
//--------------------------------------------------------
#if defined(MH_BUS_SLAVE)	//  
	str_out[1] = test_add;
#endif
#if defined(MH_BUS_MASTER)	//  
	str_out[1] = add;
#endif
//-------------------------------------------------------
	str_out[2] = com;
	str_out[3] = reg;
	
	for(BYTE i=4; i<8; i++)
		{
			str_out[i] = *str;
			str++;
		}
	
	for(BYTE i=1; i<8; i++)
		{
			crc += str_out[i];
		}
	
	str_out[8] = (BYTE)(crc >> 8);
	str_out[9] = (BYTE)(crc & 0x00FF);

//--------------------------------------------------------
#if defined(MH_BUS_RS485_ON)		
	RS_485_OUT;
	_delay_ms(RS_485_TIME_OUT);
#endif
//--------------------------------------------------------

	usart_mhbus_out(str_out);

//--------------------------------------------------------
#if defined(MH_BUS_RS485_ON)
	_delay_ms(RS_485_TIME_OUT);	
	RS_485_IN;
#endif
//--------------------------------------------------------

}

#if defined(MH_BUS_MASTER)	//  
//-------------------------------------------------------------------------
//	  ID      .
//
//	  ID .
//
//   TRUE  ID       .
//   FALSE  ID        .
//-------------------------------------------------------------------------

BYTE mhbus_id(UWORD id)
{
	UWORD id_slave = 0;
	
	id_slave = (((id_slave | MH_BUS_BYTE_1) << 8) | MH_BUS_BYTE_1);
	
	if(id_slave == id) return TRUE;
	
	return FALSE;
}
#endif

//     

#if defined(MH_BUS_SLAVE)

//-------------------------------------------------------------------------
//	   .
//
//	 .
//
//		BYTE add -  
//		
//-------------------------------------------------------------------------

BYTE mhbus_wr_add(BYTE add)
	{
		if((add < 0) | (add > 0x34))
		{
			return MH_BUS_ADD_OVF;
		}

		eeprom_write_byte(&eeprom_add, add);
		
		return MH_BUS_OK;
	}

//-------------------------------------------------------------------------
//	    ID     
//
//	 TRUE  ID 
//	 FALSE  ID  
//
//-------------------------------------------------------------------------

BYTE mhbus_id(void)
{
	UWORD test_id = 0;
	test_id = (((test_id | MH_BUS_BYTE_1) << 8) | MH_BUS_BYTE_2);
	
	if(test_id == MH_BUS_SLAVE_ID)
	{
		return TRUE;
	}
	return FALSE;
}

//-------------------------------------------------------------------------
//	  .
//
//	   
//
//-------------------------------------------------------------------------

	BYTE mhbus_rd_add(void)
	{
		return eeprom_read_byte(&eeprom_add);
	}
	
#endif

#endif /* MHBUS_SLAVE_H_ */